@@ -34,7 +34,7 @@ from psutil import virtual_memory as vmem
 
 import pandas as pd
 import json
-from npyx.io import chan_map, read_spikeglx_meta
+from npyx.io import chan_map, read_metadata
 from sklearn.decomposition import PCA
 from sklearn.preprocessing import StandardScaler
 from npyx.corr import (ccg, StarkAbeles2009_ccg_significance, ccg_sig_stack, gen_sfc, scaled_acg)
@@ -877,7 +877,7 @@ def chan_dist(chan, chanmap):
 #    breakpoint()
     return all_dist * sign_mask
 
-def in_distance_surface(dp, dist_limit=2000 ):
+def in_distance_surface(dp, dist_limit=2000):
         """
 
         Return all units from a recording that have a peak channel that is within
@@ -898,7 +898,7 @@ def in_distance_surface(dp, dist_limit=2000 ):
         chan_dist_from_top_unit = chan_dist(surface_chan, chanmap)
 
         # filter for distances that are within 2mm, so 2000micrometer from the top unit
-        chan_in_bounds = chanmap[np.abs(chan_dist_from_top_unit) < 2000][:,0]
+        chan_in_bounds = chanmap[np.abs(chan_dist_from_top_unit) < dist_limit][:,0]
 
         # get the units that are on channels within the 2mm boundary
         select_chan_in_bound = peak_chan_depth[:,0][np.isin(peak_chan_depth[:,1], chan_in_bounds)]
@@ -966,7 +966,7 @@ def chan_spread(all_wav, chan_path, unit, n_chans = 20, chan_spread_dist = 25.6)
     # on the probe version being used
 
     # get the probe version
-    probe_v = read_spikeglx_meta(chan_path)['probe_version']
+    probe_v = read_metadata(chan_path)['probe_version']
     if probe_v in ['3A', '1.0_staggered']:
         chan_spread_dist = 25.61249695
 
